import delve.io;
import delve.stream;
import delve.logging;


void file_test() {
    log_info("testing seeking streams and files");

    //
    // set up files and streams
    //

    string path = "/tmp/junk.data";

    auto appender = new AppendFile(path);
    appender.truncate(0);

    auto reader = new ReadFile(path);

    auto a_stream = new OutPositionStream(appender);
    auto r_stream = new InSeekStream(reader);

    //
    // append a bunch of stuff to file, building indexes to it
    //

    long[] data;
    data.length = 10_000;

    ulong[] positions;
    positions.length = 10;

    log_info("appending data to file");
    for (int n = 0; n < positions.length; ++n) {

        for (int i = 0; i < data.length; ++i) {
            data[i] = n * i;
        }

        positions[n] = a_stream.position;
        a_stream.put(data);
    }
    a_stream.flush;


    //
    // hop about in the file reading stuff
    //

    // straight read without seeking
    log_info("read from start of file");
    for (int n = 0; n < positions.length; ++n) {
        auto stuff = r_stream.get!(long[])();

        assert(stuff.length == data.length);
        for (int i = 0; i < stuff.length; ++i) {
            assert(stuff[i] == n * i);
        }
    }

    // seek and read
    log_info("random-access read");
    r_stream.seek(positions[3]);
    long[] stuff = r_stream.get!(long[])();
    assert(stuff.length == data.length);
    for (int i = 0; i < stuff.length; ++i) {
        assert(stuff[i] == 3 * i);
    }
    

    // small seeks and read
    log_info("small seeks and read");
    r_stream.seek(positions[5]);
    uint qty = r_stream.get!(uint)();
    long val = r_stream.get!(long)();
    log_info("got qty=%s val=%s, expected=%s,%s", qty, val, data.length, 0);
    assert(qty == data.length && val == 0);

    log_info("seeking 10 longs forward");
    r_stream.seek(r_stream.position + val.sizeof * 10);
    val = r_stream.get!(long)();
    log_info("got val=%s, expected=%s", val, 55);
    assert(val == 55);
}

int main(char[][] args) {
    log_info("starting");
    file_test();
    log_info("finished");
    return 0;
}
