module bedrock.maxim.test.selector_test;

import delve.logging;
import delve.io;
import delve.concurrency;


void doConsumer(PipeReadable reader1, PipeReadable reader2, PipeReadable reader3, PipeWriteable writer) {
    myName = "consumer";
    auto selector = new Selector();

    // handle input on reader 1 or 2
    void receive(ISelectableReadable reader, bool error) {
        log_info("got input on a reader - echoing on writer");
        ubyte data;
        reader.read(&data, data.sizeof);
        writer.write(&data, data.sizeof);
    }

    // handle input on reader 3
    void finalize(ISelectableReadable reader, bool error) {
        log_info("got input on reader3 - terminating");
        ubyte data;
        reader.read(&data, data.sizeof);

        ubyte zero;
        writer.write(&zero, zero.sizeof);
        selector.removeReadable(reader1);
        selector.removeReadable(reader2);
        selector.removeReadable(reader3);
    }

    log_info("started");
    selector.addReadable(reader1, &receive);
    selector.addReadable(reader2, &receive);
    selector.addReadable(reader3, &finalize);

    while (selector.process) {}
    log_info("terminated");
}


int main() {
    myName = "main";

    log_info("starting selector_test");
    auto pipe1 = new shared(Pipe)();
    auto pipe2 = new shared(Pipe)();
    auto pipe3 = new shared(Pipe)();
    auto pipe4 = new shared(Pipe)();
    log_info("created pipes");
    spawn(&doConsumer, "consumer", pipe1.readable, pipe2.readable, pipe3.readable, pipe4.writeable);
    log_info("spawned consumer");

    auto writeable1 = pipe1.writeable;
    auto writeable2 = pipe2.writeable;
    auto writeable3 = pipe3.writeable;
    auto readable   = pipe4.readable;

    ubyte[] items = [1,2,3,4];
    foreach (item; items) {

        log_info("writing item %s to pipe1", item);
        writeable1.write(&item, item.sizeof);
        log_info("getting an item");
        ubyte got1;
        readable.read(&got1, got1.sizeof);
        assert(got1 == item);

        log_info("writing item %s to pipe2", item);
        writeable2.write(&item, item.sizeof);
        log_info("getting an item");
        ubyte got2;
        readable.read(&got2, got2.sizeof);
        assert(got2 == item);
    }
    log_info("all done");
    ubyte zero;
    writeable3.write(&zero, zero.sizeof);
    return 0;
}
