module bedrock.maxim.test.stream_test;

import delve.stream;
import delve.logging;
import delve.io;
import delve.concurrency;

import std.conv;


class MyStreamable {
    int value;

    this(InStream stream) {
        value = stream.get!int;
    }

    this(int data) {
        value = data;
    }

    override string toString() {
        return to!(string)(value);
    }

    OutStream write(OutStream out_stream) const {
        return out_stream(value);
    }
}

void memory_test() {
    log_info("testing memory streams");
    auto output = new OutMemoryStream();

    int one = 1, two = 2, three = 3;
    byte[] four = [4, 5, 6, 7];
    auto five = new MyStreamable(8);
    int[string] six;
    
    six["hello"] = 9;
    six["sailor"] = 10;

    output(one)(two)(three)(four)(five)(six);

    auto input = new InMemoryStream(output.data);
    auto a = input.get!(int)();
    auto b = input.get!(int)();
    auto c = input.get!(int)();
    auto d = input.get!(byte[])();
    auto e = input.get!(MyStreamable)();
    auto f = input.get!(int[string])();

    log_info("sent a=%s b=%s c=%s d=%s e=%s f=%s", one, two, three, four, five, six);
    log_info("got  a=%s b=%s c=%s d=%s e=%s f=%s", a, b, c, d, e, f);
    assert(a == one &&
           b == two &&
           c == three &&
           d == four &&
           five.value == e.value &&
           six["hello"] == f["hello"]);
}

void buffered_test() {
    log_info("testing buffered streams (using Pipe)");

    auto pipe   = new shared(Pipe)();
    auto output = new OutBufferedStream(pipe.writeable, 16);
    auto input  = new InBufferedStream(pipe.readable, 8);

    int one = 1, two = 2, three = 3;
    byte[] four = [4, 5, 6, 7];
    auto five = new MyStreamable(8);
    int[string] six;
    
    six["hello"] = 9;
    six["sailor"] = 10;

    output(one)(two)(three)(four)(five)(six);
    output.flush();

    auto a = input.get!(int)();
    auto b = input.get!(int)();
    auto c = input.get!(int)();
    auto d = input.get!(byte[])();
    auto e = input.get!(MyStreamable)();
    auto f = input.get!(int[string])();

    log_info("sent a=%s b=%s c=%s d=%s e=%s f=%s", one, two, three, four, five, six);
    log_info("got  a=%s b=%s c=%s d=%s e=%s f=%s", a, b, c, d, e, f);
    assert(a == one &&
           b == two &&
           c == three &&
           d == four &&
           five.value == e.value &&
           six["hello"] == f["hello"]);
}

int main(char[][] args) {
    log_info("starting");
    memory_test();
    buffered_test();
    log_info("finished");
    return 0;
}
